#!/usr/bin/perl -wT
# $Id: vogt.pl,v 1.11 2013/10/31 06:30:38 sqko Exp $
$ENV{'BASH_ENV'} = '/bin:/usr/bin:/usr/local/bin';
$ENV{'ENV'} = '/bin:/usr/bin:/usr/local/bin';
use strict;
use warnings;
use RFC::Rig;
use RFC::Main;
use RFC::Header;
use POSIX;
use SemaforeFile;


my $type = 'mailfile';
if (@ARGV < 1)
{
    my $string = "Not enough input parameters for vogt.pl\n";
    $string = $string.'Usage: vogt.pl  Rig_number  [opt: Voltage_limit]'."\n";
    $string = $string."If no voltage limit is given, the value in the configuration\nfile for the rig is used. If this does not exsist, 600 mV is used!\n";
    die "$string";
}

my $rig = RFC::Rig->new(shift);
$rig->check_run;

#### Do not do anything durring i-V curves ####
exit if ($rig->is_iv);

my $currenttest = $rig->session;
my $limit_conf = 0;
if (@ARGV < 1)
{
    $limit_conf = $rig->get_cv('control_logic','voltage_limit_vogt');
    if (!$limit_conf)
    {
	print "Using 600 mV as default voltage limit\n";
	$limit_conf = 600;
    }
}
my $vlimit = 0;
if (@ARGV)
{
    my $tvoltage = shift;
    $tvoltage =~ /([\d\.]+)/;
    $vlimit = $1;
    $vlimit += 0; 
}
else
{
    $vlimit = $limit_conf;
}
my $elimit= $rig->get_cv('control_logic','electrolysis_limit_vogt');
$elimit = 1e30 unless defined($elimit);
my $time = localtime;
my $homedir = '/home/rig'.$rig->name.'/';
my $voltlabel = $rig->get_cv('control_logic','voltage_label_vogt');
$voltlabel = 'cell_voltage' unless ($voltlabel);  ## Default value for backwards compatibility ###
my $device = $rig->simplechannel($voltlabel);
$rig->die("No device id found for simple channel '$voltlabel'") unless ($device);

my $datadir = $homedir . $rig->name . 'test'. $currenttest;
sleep(4);
my $lockfile = SemaforeFile->new('/tmp/'.$$.'.txt',$datadir.'/lockfile_vogt');
my $res = $lockfile->lock_sh_nb();
exit unless ($res);

my @vals = ();
foreach (0..4)
{
    my $v = $device->read();
    push @vals,$v if (defined($v));
}
unless (@vals)
{
    $rig->warning_mail($type,"Could not determine cell voltage for rig $rig at $time");
    exit;
}
### Get median value ### 
@vals = sort {$a <=> $b} @vals;
my $var = @vals[POSIX::floor(@vals/2+0.6)];

if ( $var < $vlimit || $var > $elimit) 
{
    unless ($rig->check_warning($type))
    {
	my $str = $rig->get_cv('main','iv_control_names');
	if ($str)
	{
	    foreach my $psu (split(/\s*\,\s*/,$str))
	    {
		my $dev = $rig->get_device('PSU',$psu);
		$dev->current('ocv',$0);
	    }
	}
	else
	{
	    my @list = $rig->list_devices_control('PSU');
	    foreach my $psu (@list)
	    {
		my $dev = $rig->get_device('PSU',$psu);
		$dev->current('ocv',$0);
	    }
	}
	my $string = "Voltage outside range of ($vlimit - $elimit) mV reached for rig ".$rig->name." at $time\n";
	$string = $string."measured voltage was $var\n";
	if ($rig->get_cv('control_logic','additional_voltage_trip_program'))
	{
	    my $cmd = $RFC::Header::libdir.'logprograms/'.$rig->get_cv('control_logic','additional_voltage_trip_program');
	    if (-e $cmd && -x $cmd)
	    {
		$string .= 'Executing additional voltage trip program: '.$cmd;
		system($cmd.' '.$rig->name);
	    }
	}
	$rig->warning_mail($type,$string);
    }
    else
    {
	### Use direct device instance to awoid calling proglog excessively ####
	my $str = $rig->get_cv('main','iv_control_names');
	if ($str)
	{
	    foreach my $psu (split(/\s*\,\s*/,$str))
	    {
		my $dev = $rig->get_device('PSU',$psu);
		$dev->current('ocv',$0,'nolog');		    
	    }
	}
	else
	{
	    my @list = $rig->list_devices_control('PSU');
	    foreach my $psu (@list)
	    {
		my $dev = $rig->get_device('PSU',$psu);
		$dev->current('ocv',$0,'nolog');
	    }
	}
    }
}
